﻿using System;
using System.IO;
using System.Linq;
using System.Reflection;

public static class AssemblyExtensions
{
    public static Stream GetEmbeddedResourceByName(this Assembly assembly, string name)
    {
        return GetEmbeddedResourceByName(assembly, name, false);
    }

    public static Stream GetEmbeddedResourceByName(this Assembly assembly, string name, bool caseSensitive)
    {
        Func<string, bool> predicate = null;

        name = name.Replace("/", ".");

        if (caseSensitive)
        {
            predicate = x => x.Contains(name);
        }
        else
        {
            predicate = x => x.ToLower().Contains(name.ToLower());
        }

        var completeName = assembly.GetManifestResourceNames().Where(predicate).FirstOrDefault();

        if (string.IsNullOrEmpty(completeName))
        {
            return null;
        }

        return assembly.GetManifestResourceStream(completeName);
    }
}
