﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition.Primitives;
using System.Linq;
using Microsoft.Practices.Unity;

namespace MachinaAurum.Unity.Mef
{
    public class UnityAggregatedCatalog : ComposablePartCatalog
    {
        public IEnumerable<ComposablePartCatalog> Catalogs { get; set; }
        public IUnityContainer Container { get; set; }

        IList<ComposablePartDefinition> UnityParts;

        public UnityAggregatedCatalog(IUnityContainer container, params ComposablePartCatalog[] catalogs)
        {
            Container = container;
            Catalogs = catalogs;

            UnityParts = new List<ComposablePartDefinition>();
        }

        public override IEnumerable<System.Tuple<ComposablePartDefinition, ExportDefinition>> GetExports(ImportDefinition definition)
        {
            var exports = base.GetExports(definition);
            return exports;
        }

        public override IQueryable<ComposablePartDefinition> Parts
        {
            get
            {
                var originalParts = Catalogs.SelectMany(catalog => catalog.Parts).Select(part => new UnityPartDefinition(this, part)).Cast<ComposablePartDefinition>().AsQueryable();
                var unityParts = UnityParts;

                return originalParts.Union(unityParts).AsQueryable();
            }
        }

        public void AddPart(Type type, object instance)
        {
            UnityParts.Add(new UnityInstancePartDefinition(type, instance));
        }
    }
}
