﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition.Primitives;
using System.Linq;

namespace MachinaAurum.Unity.Mef
{
    class UnityInstanceExportDefinition : ExportDefinition
    {
        public object Instance;
        Type Type;

        IDictionary<string, object> MetadataDic;

        public UnityInstanceExportDefinition( Type type, object instance )
        {
            Type = type;
            Instance = instance;

            MetadataDic = new Dictionary<string, object>();
            MetadataDic.Add("ExportTypeIdentity", type.FullName);
        }

        public override IDictionary<string, object> Metadata
        {
            get
            {
                return MetadataDic;
            }
        }
        
        public override string ContractName
        {
            get
            {
                return Type.FullName;
            }
        }
    }

    class UnityInstancePart : ComposablePart
    {
        UnityInstanceExportDefinition Export;

        public UnityInstancePart(UnityInstanceExportDefinition export)
        {
            Export = export;            
        }

        public override IEnumerable<ExportDefinition> ExportDefinitions
        {
            get { yield return Export; }
        }

        public override IEnumerable<ImportDefinition> ImportDefinitions
        {
            get { return Enumerable.Empty<ImportDefinition>(); }
        }

        public override object GetExportedValue(ExportDefinition definition)
        {
            return Export.Instance;
        }

        public override void SetImport(ImportDefinition definition, System.Collections.Generic.IEnumerable<Export> exports)
        {
        }
    }

    class UnityInstancePartDefinition : ComposablePartDefinition
    {
        UnityInstanceExportDefinition Export;

        public UnityInstancePartDefinition(Type type, object instance)
        {
            Export = new UnityInstanceExportDefinition(type, instance);
        }

        public override IEnumerable<ExportDefinition> ExportDefinitions
        {
            get { yield return Export; }
        }

        public override IEnumerable<ImportDefinition> ImportDefinitions
        {
            get { return Enumerable.Empty<ImportDefinition>(); }
        }

        public override ComposablePart CreatePart()
        {
            return new UnityInstancePart(Export);
        }
    }
}
