﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition.Primitives;
using System.Linq;

namespace MachinaAurum.Unity.Mef
{
    public class UnityPart : ComposablePart
    {
        ComposablePartDefinition Part { get; set; }
        UnityAggregatedCatalog Catalog { get; set; }
        ComposablePart ComposablePart { get; set; }
        IEnumerable<ExportDefinition> Exports { get; set; }
        bool CreateOnUnity { get; set; }

        public UnityPart(UnityAggregatedCatalog catalog, ComposablePartDefinition part)
        {
            Part = part;

            if (Part != null)
            {
                ComposablePart = Part.CreatePart();
            }

            Catalog = catalog;
            CreateOnUnity = false;
        }

        public UnityPart(UnityAggregatedCatalog catalog, ExportDefinition export)
        {
            Catalog = catalog;
            CreateOnUnity = false;
            Exports = Enumerable.Range(0, 1).Select(i => export);
        }

        public override IEnumerable<ExportDefinition> ExportDefinitions
        {
            get
            {
                if (Part != null)
                {
                    return Part.ExportDefinitions;
                }
                else
                {
                    return Exports;
                }
            }
        }

        public override IEnumerable<ImportDefinition> ImportDefinitions
        {
            get
            {
                if (Part != null)
                {
                    return Part.ImportDefinitions;
                }
                else
                {
                    return Enumerable.Empty<ImportDefinition>();
                }

            }
        }

        public object Create(ComposablePartDefinition composablePart, ExportDefinition export)
        {
            if (Part != null)
            {
                object createdItem = null;
                
                try
                {
                    ComposablePart.Activate();
                    createdItem = ComposablePart.GetExportedValue(export);
                }
                catch
                { }

                if (createdItem == null)
                {
                    Type type = AppDomain.CurrentDomain.GetAssemblies()
                        .Select ( x => x.GetType((ComposablePart as ICompositionElement).DisplayName))
                        .Where ( x => x != null )
                        .SingleOrDefault();

                    if (type != null)
                    {
                        return Catalog.Container.Resolve(type, string.Empty);
                    }
                    else
                    {
                        throw new NotImplementedException();
                    }
                }
                else
                {
                    Catalog.Container.BuildUp(createdItem.GetType(), createdItem, string.Empty);
                }
                
                return createdItem;
            }
            else
            {
                Type type = Type.GetType(export.ContractName);
                
                if (type != null)
                {
                    return Catalog.Container.Resolve(type, string.Empty);
                }
                else
                {
                    throw new NotImplementedException();
                }
            }
        }

        public override object GetExportedValue(ExportDefinition definition)
        {
            return Create(Part, definition);
        }


        public override void SetImport(ImportDefinition definition, IEnumerable<Export> exports)
        {
            ComposablePart.SetImport(definition, exports);
        }
    }
}
