﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using System.Linq.Expressions;
using System.Runtime.CompilerServices;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace MachinaAurum.Xna.SlimDx.Tests
{
    public struct VertexShaderInput
    {
        public Color Color { get; set; }
        public Vector3 Position { get; set; }
    }

    public struct PixexShaderInput
    {
        public Vector4 Position { get; set; }
        public Color Color { get; set; }
    }

    public class GlobalVariable
    {
        public string Name;
        public string Type;
    }

    public class Color
    {
        public Color( float red, float green, float blue, float alpha)
        {

        }

        public float R { get; set; }
        public float G { get; set; }
        public float B { get; set; }
        public float Alpha { get; set; }
    }

    public class Vector3
    {        
    }

    public class Vector4
    {
        public Vector4( Vector3 vector, float w )
        {

        }

        public static Vector4 operator * (Vector4 a, Matrix4x4 b)
        {
            throw new NotImplementedException();
        }
    }

    public class Matrix4x4
    {
        public static Matrix4x4 operator *(Matrix4x4 a, Matrix4x4 b)
        {
            throw new NotImplementedException();
        }
    }

    public class HlslExpressionVisitor : ExpressionVisitor
    {
        Queue<string> parameters = new Queue<string>();

        public string Extract(Expression node)
        {
            Visit(node);

            var result = parameters.Dequeue();
            return result;
        }

        protected override Expression VisitConstant(ConstantExpression node)
        {
            parameters.Enqueue(node.Value.ToString());
            
            return base.VisitConstant(node);
        }

        IEnumerable<string> Parameters (NewExpression node)
        {
            foreach (var item in node.Arguments)
            {
                Visit(item);
                yield return parameters.Dequeue();
            }
        }

        protected override Expression VisitNew(NewExpression node)
        {
            var p = Parameters(node).ToArray();
            
            var e = string.Format("{0} ( {1} )", GetHlslType(node.Type.Name), string.Join(", ", p));

            parameters.Enqueue(e);

            return node;
        }

        protected override Expression VisitBinary(BinaryExpression node)
        {
            string operation = string.Empty;
            switch (node.NodeType)
            {
                case ExpressionType.Multiply:
                    operation = "mul";
                    break;

            }

            base.Visit(node.Left);
            var left = parameters.Dequeue();
            base.Visit(node.Right);            
            var right = parameters.Dequeue();

            var e = string.Format("{0}( {1}, {2} )", operation, left, right );

            parameters.Enqueue(e);

            return node;
        }

        private string GetHlslType(string name)
        {
            if (name.ToLower() == "matrix4x4") return "float4x4";
            if (name.ToLower() == "vector4") return "float4";
            if (name.ToLower() == "color") return "float4";

            throw new NotImplementedException();
        }

        protected override Expression VisitParameter(ParameterExpression node)
        {
            parameters.Enqueue(node.Name);

            return node;
        }

        protected override Expression VisitMember(MemberExpression node)
        {
            var atts = node.Member.ReflectedType.GetCustomAttributes(true).OfType<CompilerGeneratedAttribute>();

            if (atts.Any() == false)
            {
                base.VisitMember(node);
            }


            if (parameters.Count > 0)
            {
                var all = DequeueAll().Union ( new[] { node.Member.Name } ) .ToArray();
                parameters.Enqueue(string.Join(".", all));
            }
            else
            {
                parameters.Enqueue(node.Member.Name);
            }
            
            return node;
        }

        IEnumerable<string> DequeueAll()
        {
            while (parameters.Count > 0)
            {
                yield return parameters.Dequeue();
            }
        }
    
    }

    public class VertexShaderQueryProvider : IQueryProvider
    {
        ICollection<GlobalVariable> GlobalVariables;
        IQueryable Query;
        Type PixelShaperType;

        StringBuilder Code;

        public VertexShaderQueryProvider(IQueryable query, Type pixelShaperType, StringBuilder str)
        {
            Query = query;
            GlobalVariables = new List<GlobalVariable>();
            PixelShaperType = pixelShaperType;

            Code = str;
        }

        bool INVertexShader = false;
        bool VertexShaderEnd = false;
        bool INPixelShader = false;

        #region IQueryProvider Members

        public IQueryable<TElement> CreateQuery<TElement>(System.Linq.Expressions.Expression expression)
        {
            try
            {
                var query = GlobalVariableInitialization(expression);
                return query as IQueryable<TElement>;
            }
            catch
            {
                if (INVertexShader == false)
                {
                    Code.AppendLine("");
                    Code.AppendLine("PixelShaderInput VS( VertexShaderInput input )");
                    Code.AppendLine("{");

                    INVertexShader = true;
                }
                else if ((VertexShaderEnd) && (INPixelShader == false))
                { 
                    Code.AppendLine("");
                    Code.AppendLine("float4 PS( PixelShaderInput input ) : SV_Target");
                    Code.AppendLine("{");

                    INPixelShader = true;
                }

                dynamic e = expression;

                if (e.Arguments[1].Operand.Body.Type != PixelShaperType)
                {
                    var a = e.Arguments[1].Operand.Body.Arguments[1];
                    var b = e.Arguments[1].Operand.Body.Members[1].Name;
                    var c = GetHlslType(e.Arguments[1].Operand.Body.Members[1].PropertyType.Name);

                    Code.AppendFormat("{0} {1} = {2};{3}", c, b, new HlslExpressionVisitor().Extract(a), Environment.NewLine);
                }
                else
                {
                    Code.AppendLine("");
                    string name = "pixelShaderInput";
                    Code.AppendFormat("{0} {1};{2}", PixelShaperType.Name, name, Environment.NewLine);

                    if (e.Arguments[1].Operand.Body.NodeType == ExpressionType.MemberAccess)
                    {
                        Code.AppendFormat("return {0};{1}", e.Arguments[1].Operand.Body.Member.Name, Environment.NewLine);
                        Code.AppendLine(@"
        technique10 Render
        {
            pass P0
            {
                SetVertexShader( CompileShader( vs_4_0, VS() ) );
                SetGeometryShader( NULL );
                SetPixelShader( CompileShader( ps_4_0, PS() ) );
            }
        }");
                    }
                    else
                    {

                        foreach (var binding in e.Arguments[1].Operand.Body.Bindings)
                        {
                            Code.AppendFormat("{0}.{1} = {2};{3}", name, binding.Member.Name, binding.Expression.Member.Name, Environment.NewLine);
                        }

                        Code.AppendLine("return pixelShaderInput;");
                        Code.AppendLine("}");

                        VertexShaderEnd = true;
                    }
                }

                var type = typeof(VertexShader<,>).MakeGenericType(e.Arguments[1].Operand.ReturnType, PixelShaperType);
                var query = Activator.CreateInstance(type, this);
                return query;
            }
        }

        private string GetHlslType(string name)
        {
            if (name.ToLower() == "matrix4x4") return "float4x4";
            if (name.ToLower() == "vector4") return "float4";
            if (name.ToLower() == "color") return "float4";

            throw new NotImplementedException();
        }

        private dynamic GlobalVariableInitialization(System.Linq.Expressions.Expression expression)
        {
            dynamic e = expression;
            var a = e.Arguments[1].Operand.Body.Arguments[1].Member.Name;
            var b = e.Arguments[1].Operand.Body.Arguments[1].Member.PropertyType.Name;

            GlobalVariables.Add(new GlobalVariable()
            {
                Name = a,
                Type = b
            });

            Code.AppendFormat("{0} {1};{2}", a, b, Environment.NewLine);

            var type = typeof(VertexShader<,>).MakeGenericType(e.Arguments[1].Operand.ReturnType, PixelShaperType);
            var query = Activator.CreateInstance(type, this);
            return query;
        }

        public IQueryable CreateQuery(System.Linq.Expressions.Expression expression)
        {
            throw new NotImplementedException();
        }

        public TResult Execute<TResult>(System.Linq.Expressions.Expression expression)
        {
            throw new NotImplementedException();
        }

        public object Execute(System.Linq.Expressions.Expression expression)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    public class PixelShaderQueryProvider : IQueryProvider
    {
        ICollection<GlobalVariable> GlobalVariables;
        IQueryable Query;
        Type PixelShaperType;

        StringBuilder Code;

        public PixelShaderQueryProvider(IQueryable query, Type pixelShaperType, StringBuilder str)
        {
            Query = query;
            GlobalVariables = new List<GlobalVariable>();
            PixelShaperType = pixelShaperType;

            Code = str;
        }

        bool InShader = false;

        #region IQueryProvider Members

        public IQueryable<TElement> CreateQuery<TElement>(System.Linq.Expressions.Expression expression)
        {
            try
            {
                var query = GlobalVariableInitialization(expression);
                return query as IQueryable<TElement>;
            }
            catch
            {
                if (InShader == false)
                {
                    Code.AppendLine("");
                    Code.AppendLine("float4 PS( PixelShaderInput input ) : SV_Target");
                    Code.AppendLine("{");

                    InShader = true;
                }

                dynamic e = expression;

                var body = e.Arguments[1].Operand.Body;
                var nodeType = body.NodeType;
                if (nodeType == ExpressionType.Parameter)
                {
                    throw new NotSupportedException("Pixel Shader must return a float4 equivalent type.");
                }
                else if (nodeType == ExpressionType.MemberAccess)
                {
                    var a = body.Arguments[1];
                    var b = body.Members[1].Name;
                    var c = GetHlslType(body.Members[1].PropertyType.Name);

                    Code.AppendFormat("{0} {1} = {2};{3}", c, b, new HlslExpressionVisitor().Extract(a), Environment.NewLine);
                }

//                    var a = e.Arguments[1].Operand.Body.Arguments[1];
//                    var b = e.Arguments[1].Operand.Body.Members[1].Name;
//                    var c = GetHlslType(e.Arguments[1].Operand.Body.Members[1].PropertyType.Name);

//                    Code.AppendFormat("{0} {1} = {2};{3}", c, b, new HlslExpressionVisitor().Extract(a), Environment.NewLine);
//                }
//                else
//                {
//                    Code.AppendLine("");

//                    if (e.Arguments[1].Operand.Body.NodeType == ExpressionType.MemberAccess)
//                    {
//                        var a = e.Arguments[1].Operand.Body;
//                        Code.AppendFormat("return {0};{1}", new HlslExpressionVisitor().Extract(a), Environment.NewLine);
//                        Code.AppendLine("}");
//                        Code.AppendLine(@"
//        technique10 Render
//        {
//            pass P0
//            {
//                SetVertexShader( CompileShader( vs_4_0, VS() ) );
//                SetGeometryShader( NULL );
//                SetPixelShader( CompileShader( ps_4_0, PS() ) );
//            }
//        }");
//                    }                    
//                }

                var type = typeof(VertexShader<,>).MakeGenericType(e.Arguments[1].Operand.ReturnType, PixelShaperType);
                var query = Activator.CreateInstance(type, this);
                return query;
            }
        }

        private string GetHlslType(string name)
        {
            if (name.ToLower() == "matrix4x4") return "float4x4";
            if (name.ToLower() == "vector4") return "float4";
            if (name.ToLower() == "color") return "float4";

            throw new NotImplementedException();
        }

        private dynamic GlobalVariableInitialization(System.Linq.Expressions.Expression expression)
        {
            dynamic e = expression;
            var a = e.Arguments[1].Operand.Body.Arguments[1].Member.Name;
            var b = e.Arguments[1].Operand.Body.Arguments[1].Member.PropertyType.Name;

            GlobalVariables.Add(new GlobalVariable()
            {
                Name = a,
                Type = b
            });

            Code.AppendFormat("{0} {1};{2}", a, b, Environment.NewLine);

            var type = typeof(VertexShader<,>).MakeGenericType(e.Arguments[1].Operand.ReturnType, PixelShaperType);
            var query = Activator.CreateInstance(type, this);
            return query;
        }

        public IQueryable CreateQuery(System.Linq.Expressions.Expression expression)
        {
            throw new NotImplementedException();
        }

        public TResult Execute<TResult>(System.Linq.Expressions.Expression expression)
        {
            throw new NotImplementedException();
        }

        public object Execute(System.Linq.Expressions.Expression expression)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    public class PixelShader<TPixel> : IQueryable<TPixel>
    {
        StringBuilder Code;

        public PixelShader( StringBuilder code)
        {
            Code = code;
            Provider = new PixelShaderQueryProvider(this, typeof(Color), Code );
            Expression = System.Linq.Expressions.Expression.Constant(this);
        }

        #region IEnumerable<TPixel> Members

        public IEnumerator<TPixel> GetEnumerator()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IQueryable Members

        public Type ElementType { get; private set; }

        public Expression Expression { get; private set; }

        public IQueryProvider Provider { get; private set; }

        #endregion
    }

    public class VertexShader<TVertex, TPixel> : IQueryable<TVertex>
    {
        StringBuilder Code;

        public VertexShader( StringBuilder code )
        {
            Code = code;
            Provider = new VertexShaderQueryProvider(this, typeof(TPixel), Code);
        }

        public VertexShader(IQueryProvider queryProvider)
        {
            Provider = queryProvider;
        }

        #region IEnumerable<T> Members

        public IEnumerator<TVertex> GetEnumerator()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IQueryable Members

        public Type ElementType
        {
            get { throw new NotImplementedException(); }
        }

        public System.Linq.Expressions.Expression Expression
        {
            get { return System.Linq.Expressions.Expression.Constant(this); }
        }

        public IQueryProvider Provider { get; set; }

        #endregion
    }

    class HlslBuilder<TVertex, TPixel>
    {
        StringBuilder Code;
        public HlslBuilder()
        {
            Code = new StringBuilder();

            VertexShader = new VertexShader<TVertex, TPixel>(Code);
            PixelShader = new PixelShader<TPixel>(Code);
        }

        public IQueryable<TVertex> VertexShader { get; set; }
        public IQueryable<TPixel> PixelShader { get; set; }

        public Matrix4x4 World { get; set; }
        public Matrix4x4 View { get; set; }
        public Matrix4x4 Projection { get; set; }

        public string GetHlsl()
        {
            return Code.ToString();
        }
    }

    [Microsoft.VisualStudio.TestTools.UnitTesting.TestClass]
    public class HlslBuilderTests
    {
        [Microsoft.VisualStudio.TestTools.UnitTesting.TestMethod]
        public void MyTestMethod()
        {
            var builder = new HlslBuilder<VertexShaderInput, PixexShaderInput>();
            //var vs = from input in builder.VertexShader
            //         let world = builder.World
            //         let view = builder.View
            //         let projection = builder.Projection

            //         let worldViewProj = world * view * projection
            //         let position = new Vector4 ( input.Position, 1.0f) * worldViewProj
            //         let color = new Color ( input.Color.R , input.Color.G, input.Color.B, 1.0f )

            //         select new PixexShaderInput()
            //         {
            //             Position = position,
            //             Color = color
            //         };

            
            var ps = from input in builder.PixelShader
                     let color = new Color ( input.Color.R , input.Color.G, input.Color.B, 1.0f )
                     select color;

            var hlsl = builder.GetHlsl();

            Assert.AreEqual(@"World Matrix4x4;
View Matrix4x4;
Projection Matrix4x4;

PixelShaderInput VS( VertexShaderInput input )
{
float4x4 worldViewProj = mul( mul( world, view ), projection );
float4 position = mul( float4 ( input.Position, 1 ), worldViewProj );
float4 color = float4 ( input.Color.R, input.Color.G, input.Color.B, 1 );

PixexShaderInput pixelShaderInput;
pixelShaderInput.Position = position;
pixelShaderInput.Color = color;
return pixelShaderInput;
}


      float4 PS( PixelShaderInput input ) : SV_Target
        {
            return input.Color;
        }

        technique10 Render
        {
            pass P0
            {
                SetVertexShader( CompileShader( vs_4_0, VS() ) );
                SetGeometryShader( NULL );
                SetPixelShader( CompileShader( ps_4_0, PS() ) );
            }
        }
", hlsl);
        }

        [TestMethod]
        public void MyTestMethod2()
        {
            Expression<Func<PixexShaderInput,float>> a = input => input.Color.R;
            string code = new HlslExpressionVisitor().Extract(a);

            Assert.AreEqual("input.Color.R", code);
        }

        [TestMethod]
        [ExpectedException(typeof(NotSupportedException))]
        public void PixelShaderCannotReturnThePixelShaderInput()
        {
            var builder = new HlslBuilder<VertexShaderInput, PixexShaderInput>();

            var ps = from input in builder.PixelShader
                     select input;

            var code = builder.GetHlsl();

        }

        [TestMethod]
        [ExpectedException(typeof(NotSupportedException))]
        public void PixelShaderCannotReturnTypesDifferentFromColor()
        {
            var builder = new HlslBuilder<VertexShaderInput, PixexShaderInput>();

            var ps = from input in builder.PixelShader
                     let output = input.Color
                     select output;

            var code = builder.GetHlsl();

        }
    }
}
