﻿using System.Collections.Generic;
using System.Linq;
using MachinaAurum.Xna.Graphics;
using MachinaAurum.Xna.Graphics.Effects;
using SlimDX.D3DCompiler;

namespace MachinaAurum.Xna.DirectX10
{
    class DirectX10Effect : AurumEffect
    {
        public SlimDX.Direct3D10.Effect effect;
        public SlimDX.Direct3D10.EffectTechnique technique;
        public SlimDX.Direct3D10.EffectPass pass;

        DirectX10GraphicDevice Device;
        IEffectCode Code;
        List<DirectX10EffectTechnique> Tecniques;

        public DirectX10Effect(DirectX10GraphicDevice device, IEffectCode code)
        {
            Device = device;
            Tecniques = new List<DirectX10EffectTechnique>();
            Code = code;
        }

        public override void Create()
        {
            effect = SlimDX.Direct3D10.Effect.FromString(Device.Device, Code.Code, "fx_4_0", ShaderFlags.Debug | ShaderFlags.SkipOptimization, EffectFlags.None, null, null);
            technique = effect.GetTechniqueByIndex(0);
            pass = technique.GetPassByIndex(0);

            var techniques = Enumerable.Range(0, effect.Description.TechniqueCount)
                .Select(i => effect.GetTechniqueByIndex(i))
                .Select(x => new DirectX10EffectTechnique(x));

            Tecniques.AddRange(techniques);
            CurrentTechnique = Tecniques[0];
        }

        public override object Signature
        {
            get { return pass.Description.Signature; }
        }

        public override void AddVariable(string name, Microsoft.Xna.Framework.Matrix matrix)
        {
            var result = effect.GetVariableByName(name).AsMatrix().SetMatrix(matrix.AsSlimDX());
        }

        public override bool TrySetMember(System.Dynamic.SetMemberBinder binder, object value)
        {
            var variable = effect.GetVariableByName(binder.Name);

            if (variable == null)
            {
                return base.TrySetMember(binder, value);
            }

            if (value is Microsoft.Xna.Framework.Matrix)
            {
                var xnaMatrix = (Microsoft.Xna.Framework.Matrix)value;
                variable.AsMatrix().SetMatrix(xnaMatrix.AsSlimDX());
                return true;
            }

            return base.TrySetMember(binder, value);
        }
    }
}
