﻿using MachinaAurum.Xna.Graphics;
using SlimDX;
using SlimDX.Direct3D10;
using SlimDX.DXGI;

namespace MachinaAurum.Xna.DirectX10
{
    class DirectX10IndexBuffer : AurumIndexBuffer
    {
        DirectX10GraphicDevice Device { get; set; }
        byte[] Data { get; set; }

        public int IndexCount { get; set; }
        public int IndexSize { get; set; }

        SlimDX.Direct3D10.Buffer Indices { get; set; }

        public DirectX10IndexBuffer( DirectX10GraphicDevice device, int indexSize, int indexCount )
        {
            Device = device;

            IndexCount = indexCount;
            IndexSize = indexSize;
        }

        public override void SetData(byte[] data)
        {
            Data = data;
        }

        public override void Create()
        {
            using (var stream = new DataStream(Data.Length, true, true))
            {
                stream.WriteRange<byte>(Data, 0, Data.Length);
                stream.Position = 0;

                Indices = new SlimDX.Direct3D10.Buffer(Device.Device, stream, new BufferDescription()
                {
                    BindFlags = BindFlags.IndexBuffer,
                    CpuAccessFlags = CpuAccessFlags.None,
                    OptionFlags = ResourceOptionFlags.None,
                    SizeInBytes = Data.Length,
                    Usage = ResourceUsage.Default
                });
            }
        }

        public override void Draw()
        {
            var format = Format.R16_UInt;

            if (IndexSize == 32)
            {
                format = Format.R32_UInt;
            }

            Device.Device.InputAssembler.SetIndexBuffer(Indices, format, 0);
            Device.Device.DrawIndexed(IndexCount, 0, 0);
        }

        protected override void Dispose(bool cleanManaged)
        {
        }
    }
}
