﻿using System.Linq;
using System.Threading;
using MachinaAurum.Xna.Bus;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace MachinaAurum.Xna.Tests.Bus
{
    [TestClass]
    public class MemoryBusTests
    {
        [TestMethod]
        public void MemoryBusMustCallTheCommandSentEvent()
        {
            //Arrange

            bool called = false;

            var bus = new MemoryBus();
            bus.OnCommandSent += (s, e) => { called = true; };

            var command = new DummyCommand();

            //Act            

            bus.Send(command);

            //Assert

            Assert.IsTrue(called, "OnCommandSent was not called.");
        }

        [TestMethod]
        public void MemoryBusMustGenerateTheObservableSignal()
        {
            //Arrange

            var wait = new ManualResetEvent(false);

            var command = new DummyCommand();

            var bus = new MemoryBus();
            bus.Commands.Where(x => x == command).Subscribe(Observer.Create<ICommand>(x => { wait.Set(); }));

            //Act

            bus.Send(command);

            //Assert
            
            Assert.IsTrue(wait.WaitOne(10), "Observale signal was not generated.");
        }
    }
}
