﻿using MachinaAurum.Xna.Graphics.ContentTypeReaders;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Xna.Framework.Graphics;
using Rhino.Mocks;
using MachinaAurum.Xna.Graphics;

namespace MachinaAurum.Xna.SlimDx.Tests
{
    [TestClass()]
    public class DefaultAurumVertexBufferReaderTest : BaseTest
    {
        [TestMethod()]
        [Owner("dleite")]
        [Description("DefaultAurumVertexBufferReader must read all information of vertex buffer")]
        public void DefaultAurumVertexBufferReaderMustReadAllInformationOfVertexBuffer()
        {
            // Arrange

            DefaultAurumVertexBufferReader target = new DefaultAurumVertexBufferReader();

            IStreamReader fakeReader = MockRepository.GenerateMock<IStreamReader>();
            IGraphicDevice fakeDevice = MockRepository.GenerateMock<IGraphicDevice>();
            IAurumVertexBuffer fakeVertexBuffer = MockRepository.GenerateMock<IAurumVertexBuffer>();

            Devices.Graphic = fakeDevice;

            VertexDeclaration fakeVertexDeclaration = new VertexDeclaration( new VertexElement ( 0, VertexElementFormat.Vector3, VertexElementUsage.Position, 0 ));
            int fakeVertexCount = 1;
            int dataSize = fakeVertexCount * fakeVertexDeclaration.VertexStride;
            byte[] fakeData = null;

            fakeReader.Expect(x => x.ReadRawObject<VertexDeclaration>()).Return(fakeVertexDeclaration);
            fakeReader.Expect(x => x.ReadInt32()).Return(fakeVertexCount);
            fakeReader.Expect(x => x.ReadBytes(dataSize)).Return(fakeData);
            fakeDevice.Expect(x => x.CreateVertexBuffer(fakeVertexDeclaration, fakeVertexCount)).Return(fakeVertexBuffer);
            fakeVertexBuffer.Expect(x => x.SetData(fakeData));

            //Act

            var createdBuffer = target.Read(fakeReader);

            //Assert

            Assert.AreSame(createdBuffer, fakeVertexBuffer);

            fakeReader.VerifyAllExpectations();
            fakeDevice.VerifyAllExpectations();
            fakeVertexBuffer.VerifyAllExpectations();

            Devices.Graphic = null;
        }
    }
}
