﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using MachinaAurum.Unity.Mef;
using MachinaAurum.Xna.Console;
using MachinaAurum.Xna.Graphics;
using Microsoft.Practices.Unity;
using Microsoft.Xna.Framework;
using MachinaAurum.TemplateEngine;

namespace MachinaAurum.Xna
{
    public abstract class AurumGame : Game
    {
        protected AurumGraphicsDeviceManager Graphics;
        protected ServiceManager ServiceManager;

        Container Container;
        TraceSource TraceSource;        

        protected AurumGame(params Assembly[] assemblies)
        {
            TraceSource = new System.Diagnostics.TraceSource(TraceSourceNames.MachinaAurumXna);

            try
            {
                var allAssemblies = assemblies.Union(new[] { typeof(MachinaAurumXnaAssembly).Assembly }).ToArray();
                Container = new Container(allAssemblies);
                Container.RegisterInstance<Game>(this);

                Graphics = Container.Resolve<AurumGraphicsDeviceManager>();

                Container.RegisterInstance<AurumGraphicsDeviceManager>(Graphics);
                Container.RegisterType<ITemplateEngine, RazorTemplateEngine>(new ContainerControlledLifetimeManager());
                
                ServiceManager = Container.Resolve<ServiceManager>();
            }
            catch (Exception e)
            {
                TraceSource.TraceEvent(TraceEventType.Error, 0, e.ToString());
                throw;
            }
        }

        protected override void Initialize()
        {
            base.Initialize();

            if ((Devices.Graphic != null) && (this.GraphicsDevice == null))
            {
                this.LoadContent();
            }
        }
    }
}
