﻿using System;
using System.Linq;

#if WINDOWS_PHONE

using Microsoft.Phone.Reactive;

#endif

namespace MachinaAurum.Xna.Bus
{
    internal class MemoryBus : IBus, IBusObservable
    {
        public MemoryBus()
        {
            string domainEventsName = ByReflection.Event((MemoryBus x) => x.OnDomainEventSent).Name;
            string commandsName = ByReflection.Event((MemoryBus x) => x.OnCommandSent).Name;

            DomainEvents = from e in Observable.FromEvent<DomainEventSentEventArgs>(this, domainEventsName) select e.EventArgs.DomainEvent;
            Commands = from e in Observable.FromEvent<CommandSentEventArgs>(this, commandsName) select e.EventArgs.Command;
        }

        #region IBus Members

        public void Send(ICommand command)
        {
            var onCommandSent = OnCommandSent;
            if (onCommandSent != null)
            {
                onCommandSent(this, new CommandSentEventArgs() { Command = command });
            }
        }

        public void Send(IDomainEvent domainEvent)
        {
            var onDomainEventSent = OnDomainEventSent;
            if (onDomainEventSent != null)
            {
                onDomainEventSent(this, new DomainEventSentEventArgs() { DomainEvent = domainEvent });
            }
        }

        #endregion

        #region IBusObservable Members

        public event EventHandler<CommandSentEventArgs> OnCommandSent;
        public event EventHandler<DomainEventSentEventArgs> OnDomainEventSent;

        public IObservable<IDomainEvent> DomainEvents { get; private set; }
        public IObservable<ICommand> Commands { get; private set; }

        #endregion
    }
}