﻿
namespace MachinaAurum.Xna.Graphics.Effects
{
    public class PositionColorEffectCode : IEffectCode
    {
        private static string Code = @"
        matrix World;
        matrix View;
        matrix Projection;

        struct VertexShaderInput
        {
	        float3 Position : POSITION;
	        float4 Color : COLOR;
        };

        struct PixelShaderInput
        {
	        float4 Position : SV_POSITION;
	        float4 Color : COLOR0;
        };

        PixelShaderInput VS( VertexShaderInput input )
        {
	        PixelShaderInput ps;
	        
	        float4x4 worldViewProj = mul(mul(World, View), Projection);
	        ps.Position = mul( float4(input.Position,1), worldViewProj);
	        ps.Color = float4 ( input.Color.r , input.Color.g, input.Color.b, 1 );

            return ps;
        }

        float4 PS( PixelShaderInput input ) : SV_Target
        {
            return input.Color;
        }

        technique10 Render
        {
            pass P0
            {
                SetVertexShader( CompileShader( vs_4_0, VS() ) );
                SetGeometryShader( NULL );
                SetPixelShader( CompileShader( ps_4_0, PS() ) );
            }
        }";

        #region IEffectCode Members

        string IEffectCode.Code
        {
            get { return PositionColorEffectCode.Code; }
        }

        #endregion
    }

    public class PositionColorEffect : IAurumEffect
    {
        IAurumEffect Effect;

        public PositionColorEffect()
            : this(Devices.Graphic.CreateEffect(new PositionColorEffectCode()))
        {
            Effect.Create();
        }

        public PositionColorEffect(IAurumEffect effect)
        {
            Effect = effect;
        }

        #region IAurumEffect Members

        public object Signature
        {
            get { return Effect.Signature; }
        }

        public IAurumEffectTechnique CurrentTechnique
        {
            get
            {
                return Effect.CurrentTechnique;
            }
            set
            {
                Effect.CurrentTechnique = value;
            }
        }

        void IAurumEffect.Create()
        {
            Effect.Create();
        }

        void IAurumEffect.AddVariable(string name, Microsoft.Xna.Framework.Matrix matrix)
        {
            Effect.AddVariable(name, matrix);
        }

        #endregion

        public Microsoft.Xna.Framework.Matrix World
        {
            set
            {
                Effect.AddVariable("World", value);
            }
        }

        public Microsoft.Xna.Framework.Matrix View
        {
            set
            {
                Effect.AddVariable("View", value);
            }
        }

        public Microsoft.Xna.Framework.Matrix Projection
        {
            set
            {
                Effect.AddVariable("Projection", value);
            }
        }
    }
}
