﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;

namespace MachinaAurum.Xna.Graphics
{
    public class AurumModelMesh : IAurumModelMesh
    {
        private string Name;
        private BoundingSphere boundingSphere;
        private IAurumModelMeshPart[] MeshParts;
        private object Tag;
        public IAurumModelBone ParentBone { get; set; }

        public IEnumerable<IAurumEffect> Effects
        {
            get
            {
                return MeshParts.Select(x => x.Effect);
            }
        }

        public AurumModelMesh(string name, IAurumModelBone parentBone, Microsoft.Xna.Framework.BoundingSphere boundingSphere, IAurumModelMeshPart[] meshParts, object tag)
        {
            // TODO: Complete member initialization
            this.Name = name;
            this.ParentBone = parentBone;
            this.boundingSphere = boundingSphere;
            this.MeshParts = meshParts;
            this.Tag = tag;
        }

        public void Draw()
        {
            int count = this.MeshParts.Length;

            for (int i = 0; i < count; i++)
            {
                IAurumModelMeshPart part = this.MeshParts[i];
                IAurumEffect effect = part.Effect;

                if (effect == null)
                {
                    throw new InvalidOperationException("ModelHasNoEffect");
                }

                foreach (var pass in effect.CurrentTechnique.Passes)
                {
                    pass.Apply();
                    part.Draw();
                }
            }

        }
    }
}
