﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using System.Threading.Tasks;
using System.Collections.Concurrent;

namespace MachinaAurum.Xna
{
    /// <summary>
    /// Aggregate IUpdateables to be called in parallel.
    /// </summary>
    public class ParallelGameComponent : IUpdateable
    {
        public const int ThresholdToParallelize = 10;

        int UpdateOrder;
        bool Enabled;

        IList<IUpdateable> Childs;

        public ParallelGameComponent()
        {
            Childs = new List<IUpdateable>();
        }

        #region IUpdateable Members

        public event EventHandler<EventArgs> UpdateOrderChanged;
        public event EventHandler<EventArgs> EnabledChanged;
        
        public void Update(GameTime gameTime)
        {
            if (Childs.Count > ThresholdToParallelize)
            {
                RunParallel(gameTime);
            }
            else
            {
                RunSerial(gameTime);
            }            
        }

        protected virtual void RunSerial(GameTime gameTime)
        {
            foreach (var updateable in Childs)
            {
                updateable.Update(gameTime);
            }
        }

        protected virtual void RunParallel(GameTime gameTime)
        {
            var partioner = Partitioner.Create(Childs, false);
            Parallel.ForEach(partioner, (item) => item.Update(gameTime));
        }
        
        bool IUpdateable.Enabled
        {
            get { return Enabled; }
        }

        int IUpdateable.UpdateOrder
        {
            get { return UpdateOrder; }
        }
        
        #endregion

        public void Add ( IUpdateable component )
        {
            Childs.Add(component);
        }

        private void CallUpdateOrderChanged ()
        {
            var updateOrderChanged = UpdateOrderChanged;
            if (updateOrderChanged != null)
            {
                updateOrderChanged(this, EventArgs.Empty);
            }
        }

        private void CallEnabledChanged()
        {
            var enabledChanged = EnabledChanged;
            if (enabledChanged != null)
            {
                enabledChanged(this, EventArgs.Empty);
            }
        }
    }
}
