﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Reflection;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Web;

namespace MachinaAurum.WebServer
{
    [ServiceBehavior( InstanceContextMode = InstanceContextMode.Single)]
    class WebServerService : IWebServerService
    {
        [ImportMany]
        public IEnumerable<Lazy<IController, IControllerMetada>> Controllers { get; set; }

        #region IConsole Members

        public Stream Get( Message msg)
        {
            return Call(msg);
        }

        public Stream Invoke(Message msg)
        {
            return Call(msg);
        }

        #endregion

        private Stream Call(Message msg)
        {
            var a = msg.Properties["Via"] as Uri;

            var fileName = a.PathAndQuery.Replace("/Console/", string.Empty);
            var fileInfo = new FileInfo(fileName);
            var extensions = new[] { ".png", ".js", ".css" };

            if (extensions.Contains(fileInfo.Extension))
            {
                return Assembly.GetExecutingAssembly().GetEmbeddedResourceByName(fileName);
            }

            var controllerName = a.Segments.Skip(2).DefaultIfEmpty("Main").First();
            var methodName = a.Segments.Skip(3).DefaultIfEmpty("Main").First();

            var controller = Controllers.Where(x => x.Metadata.Name == controllerName).SingleOrDefault();
            var methodInfo = controller.Value.GetType().GetMethod(methodName);

            object[] parameters = null;
            
            if (msg.IsEmpty == false)
            {
                var body = msg.GetReaderAtBodyContents();
                body.Read();
                string value = body.Value.ToString();
                string content = value.DecodeBase64 (Encoding.Default);
                var dic = HttpUtility.ParseQueryString(content);

                parameters = BindParameters(methodInfo, dic).ToArray();
            }


            return methodInfo.Invoke(controller.Value, parameters) as Stream;
        }

        private IEnumerable<object> BindParameters(MethodInfo methodInfo, NameValueCollection collection)
        {
            foreach (var item in methodInfo.GetParameters())
            {
                object result = null;
                try
                {
                    result =  collection[item.Name.ToLower()];
                }
                catch
                {
                }

                yield return result;
            }
        }
    }

}
