using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using MachinaAurum.Xna;
using MachinaAurum.Xna.Graphics;
using MachinaAurum.Xna.Graphics.Effects;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using SlimDX.Direct3D10;
using MachinaAurum.Xna.Console;

namespace XnaSlimDx.SimpleTest
{
    public class Drawabletexture2D
    {
        public void Begin()
        {
            var device = Devices.Graphic as MachinaAurum.Xna.DirectX10.DirectX10GraphicDevice;

            //ID3D10RenderTargetView* renderTargets[1] = {mColorMapRTV};
            //md3dDevice->OMSetRenderTargets(1, renderTargets, mDepthMapDSV);
            device.Device.OutputMerger.SetTargets(mColorMapRTV);

            //md3dDevice->RSSetViewports(1, &mViewport);
            device.Device.Rasterizer.SetViewports(Viewport);

            //// only clear if we actually created a color map.
            if (mColorMapRTV != null)
            {
                //md3dDevice->ClearRenderTargetView(mColorMapRTV, BLACK);
                device.Device.ClearRenderTargetView(mColorMapRTV, new SlimDX.Color4(0));
            }

            //md3dDevice->ClearDepthStencilView(mDepthMapDSV, D3D10_CLEAR_DEPTH, 1.0f, 0);
            device.Device.ClearDepthStencilView(mDepthMapDSV, DepthStencilClearFlags.Depth, 1.0f, 0);

        }

        public void End()
        {
            var device = Devices.Graphic as MachinaAurum.Xna.DirectX10.DirectX10GraphicDevice;

            //if (mColorMapSRV)
            if (mColorMapSRV != null)
            {
                //    md3dDevice->GenerateMips(mColorMapSRV);
                device.Device.GenerateMips(mColorMapSRV);
            }
        }

        private void CreateDepthMap()
        {
            int mWidth = 512;
            int mHeight = 512;

            var device = Devices.Graphic as MachinaAurum.Xna.DirectX10.DirectX10GraphicDevice;

            //ID3D10Texture2D* depthMap = 0;
            SlimDX.Direct3D10.Texture2D depthMap;

            //D3D10_TEXTURE2D_DESC texDesc;
            SlimDX.Direct3D10.Texture2DDescription texDesc = new Texture2DDescription();

            //texDesc.Width = mWidth;
            texDesc.Width = mWidth;
            //texDesc.Height = mHeight;
            texDesc.Height = mHeight;
            //texDesc.MipLevels = 1;
            texDesc.MipLevels = 1;
            //texDesc.ArraySize = 1;
            texDesc.ArraySize = 1;
            //texDesc.Format = DXGI_FORMAT_R32_TYPELESS;
            texDesc.Format = SlimDX.DXGI.Format.R32_Typeless;
            //texDesc.SampleDesc.Count = 1;
            //texDesc.SampleDesc.Quality = 0;
            texDesc.SampleDescription = new SlimDX.DXGI.SampleDescription(1, 0);
            //texDesc.Usage = D3D10_USAGE_DEFAULT;
            texDesc.Usage = SlimDX.Direct3D10.ResourceUsage.Default;
            //texDesc.BindFlags = D3D10_BIND_DEPTH_STENCIL | D3D10_BIND_SHADER_RESOURCE;
            texDesc.BindFlags = SlimDX.Direct3D10.BindFlags.DepthStencil | SlimDX.Direct3D10.BindFlags.ShaderResource;
            //texDesc.CPUAccessFlags = 0;
            texDesc.CpuAccessFlags = 0;
            //texDesc.MiscFlags = 0;
            texDesc.OptionFlags = 0;

            //HR(md3dDevice->CreateTexture2D(&texDesc, 0, &depthMap));
            depthMap = new SlimDX.Direct3D10.Texture2D(device.Device, texDesc);

            //D3D10_DEPTH_STENCIL_VIEW_DESC dsvDesc;
            SlimDX.Direct3D10.DepthStencilViewDescription dsvDesc = new DepthStencilViewDescription();
            //dsvDesc.Format = DXGI_FORMAT_D32_FLOAT;
            dsvDesc.Format = SlimDX.DXGI.Format.D32_Float;
            //dsvDesc.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;
            dsvDesc.Dimension = SlimDX.Direct3D10.DepthStencilViewDimension.Texture2D;
            //dsvDesc.Texture2D.MipSlice = 0;
            dsvDesc.MipSlice = 0;

            //HR(md3dDevice->CreateDepthStencilView(depthMap, &dsvDesc, &mDepthMapDSV));
            mDepthMapDSV = new DepthStencilView(device.Device, depthMap, dsvDesc);

            //D3D10_SHADER_RESOURCE_VIEW_DESC srvDesc;
            ShaderResourceViewDescription srvDesc = new ShaderResourceViewDescription();
            //srvDesc.Format = DXGI_FORMAT_R32_FLOAT;
            srvDesc.Format = SlimDX.DXGI.Format.R32_Float;
            //srvDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
            srvDesc.Dimension = ShaderResourceViewDimension.Texture2D;
            //srvDesc.Texture2D.MipLevels = texDesc.MipLevels;
            srvDesc.MipLevels = texDesc.MipLevels;
            //srvDesc.Texture2D.MostDetailedMip = 0;
            srvDesc.MostDetailedMip = 0;
            //HR(md3dDevice->CreateShaderResourceView(depthMap, &srvDesc, &mDepthMapSRV));
            mDepthMapSRV = new ShaderResourceView(device.Device, depthMap, srvDesc);

            //// View saves a reference to the texture so we can
            //// release our reference.
            //ReleaseCOM(depthMap);

        }

        RenderTargetView mColorMapRTV;
        DepthStencilView mDepthMapDSV;
        ShaderResourceView mDepthMapSRV;
        SlimDX.Direct3D10.Viewport Viewport;
        ShaderResourceView mColorMapSRV;

        int mWidth = 512;
        int mHeight = 512;


        public Drawabletexture2D()
        {
            Viewport = new SlimDX.Direct3D10.Viewport(0, 0, mWidth, mHeight, 0, 1);
            CreateDepthMap();
            BuildColorMap();
        }

        void BuildColorMap()
        {

            var device = Devices.Graphic as MachinaAurum.Xna.DirectX10.DirectX10GraphicDevice;

            SlimDX.Direct3D10.Texture2D colorMap;

            SlimDX.Direct3D10.Texture2DDescription texDesc = new Texture2DDescription();
            texDesc.Width = mWidth;
            texDesc.Height = mHeight;
            texDesc.MipLevels = 0;
            texDesc.ArraySize = 1;
            texDesc.Format = SlimDX.DXGI.Format.R8G8B8A8_UNorm;
            texDesc.SampleDescription = new SlimDX.DXGI.SampleDescription(1, 0);
            texDesc.Usage = SlimDX.Direct3D10.ResourceUsage.Default;
            texDesc.BindFlags = SlimDX.Direct3D10.BindFlags.RenderTarget | SlimDX.Direct3D10.BindFlags.ShaderResource;
            texDesc.CpuAccessFlags = 0;
            texDesc.OptionFlags = ResourceOptionFlags.GenerateMipMaps;

            colorMap = new SlimDX.Direct3D10.Texture2D(device.Device, texDesc);


            mColorMapRTV = new RenderTargetView(device.Device, colorMap);
            mColorMapSRV = new ShaderResourceView(device.Device, colorMap);

            //// View saves a reference to the texture so we can
            //// release our reference.
            //ReleaseCOM(depthMap);
        }
    }
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : AurumGame
    {
        SpriteBatch spriteBatch;

        public Game1()
            : base(GetAssemblies().ToArray())
        {
            Graphics.Start(GraphicsDeviceApi.DirectX10);

            Content.RootDirectory = "Content";
        }

        static IEnumerable<Assembly> GetAssemblies()
        {
            yield return typeof(MachinaAurumXnaSlimDxAssembly).Assembly;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            ServiceManager.StartWebServer();
                        
            base.Initialize();
        }

        AurumModel Cube;
        Matrix[] CubeMatrices;

        PositionColorEffect PositionColorEffect;

        Drawabletexture2D texture2D;

        protected override void LoadContent()
        {
            Cube = Content.Load<AurumModel>("cube_triangulate");
            CubeMatrices = new Matrix[Cube.Bones.Count()];
            Cube.CopyAbsoluteBoneTransformsTo(CubeMatrices);


            PositionColorEffect = new PositionColorEffect();
            PositionColorEffect.World = Matrix.Identity * GetGamePadMatrix();
            PositionColorEffect.View = GetViewMatrix();
            PositionColorEffect.Projection = GetProjectionMatrix();

            texture2D = new Drawabletexture2D();
        }


        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }


        public Vector3 Angles;
        public Vector3 Translation = Vector3.Zero;
        public Vector3 Scale = Vector3.One;

        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            float rScale = GamePad.GetState(PlayerIndex.One).Buttons.RightShoulder == ButtonState.Pressed ? (-1) : (1);
            float lScale = GamePad.GetState(PlayerIndex.One).Buttons.LeftShoulder == ButtonState.Pressed ? (-1) : (1);

            Angles.X = 1.0f * (float)gameTime.ElapsedGameTime.TotalSeconds * GamePad.GetState(PlayerIndex.One).Triggers.Right * 100 * rScale;
            Angles.Y = 1.0f * (float)gameTime.ElapsedGameTime.TotalSeconds * GamePad.GetState(PlayerIndex.One).Triggers.Left * 100 * lScale;

            Translation.X = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.X * 100;
            Translation.Z = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y * -100;

            Scale.X = (GamePad.GetState(PlayerIndex.One).ThumbSticks.Right.X + 1);
            Scale.Z = (GamePad.GetState(PlayerIndex.One).ThumbSticks.Right.Y + 1);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            Devices.Graphic.Clear(Color.CornflowerBlue);


            //Scenario2();

            //texture2D.Begin();

            Scenario1();

            //texture2D.End();

            //TODO return to the original render target;
            Scenario1();


            base.Draw(gameTime);
        }

        private void Scenario2()
        {
            PositionColorEffect.World = Matrix.Identity;// GetGamePadMatrix();
            PositionColorEffect.View = GetViewMatrix();
            PositionColorEffect.Projection = GetProjectionMatrix();

            foreach (var pass in PositionColorEffect.CurrentTechnique.Passes)
            {
                pass.Apply();
            }

            Devices.Graphic.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineList, new[]
               {
                    new VertexPositionColor ( new Vector3 ( 0, 0, 0), Color.Red ),
                    new VertexPositionColor ( new Vector3 ( 100, 0, 0), Color.Red ),

                    new VertexPositionColor ( new Vector3 ( 0, 0, 0), Color.Green ),
                    new VertexPositionColor ( new Vector3 ( 0, 100, 0), Color.Green ),

                    new VertexPositionColor ( new Vector3 ( 0, 0, 0), Color.Blue ),
                    new VertexPositionColor ( new Vector3 ( 0, 0, 100), Color.Blue )
               }, 0, 3, PositionColorEffect);
        }

        private void Scenario1()
        {
            foreach (var mesh in Cube.Meshes)
            {
                foreach (dynamic effect in mesh.Effects)
                {
                    effect.World = CubeMatrices[mesh.ParentBone.Index] * GetGamePadMatrix();
                    effect.View = GetViewMatrix();
                    effect.Projection = GetProjectionMatrix();
                }

                mesh.Draw();
            }

            // floor 

            foreach (var mesh in Cube.Meshes)
            {
                foreach (dynamic effect in mesh.Effects)
                {
                    effect.World = CubeMatrices[mesh.ParentBone.Index] * Matrix.CreateTranslation(0.0f, -100.0f, 0.0f) * Matrix.CreateScale(1.0f, 0.1f, 1.0f);
                    effect.View = GetViewMatrix();
                    effect.Projection = GetProjectionMatrix();
                }

                mesh.Draw();
            }
        }

        Matrix GetGamePadMatrix()
        {
            return Matrix.CreateScale(Scale.X) * Matrix.CreateFromYawPitchRoll(Angles.X, Angles.Y, Angles.Z) * Matrix.CreateTranslation(Translation);
        }

        Matrix GetViewMatrix()
        {
            return Microsoft.Xna.Framework.Matrix.CreateLookAt(
                        cameraPosition: new Microsoft.Xna.Framework.Vector3(120, 120, 120),
                        cameraTarget: Microsoft.Xna.Framework.Vector3.Zero,
                        cameraUpVector: Microsoft.Xna.Framework.Vector3.Up);
        }

        Matrix GetProjectionMatrix()
        {
            return Microsoft.Xna.Framework.Matrix.CreatePerspectiveFieldOfView((float)Math.PI / 4.0f, 1.3f, 1, 1000);
        }
    }
}
